<?php

namespace App\Tasks;

use App\Installation\Commands\Contracts\RequiresInput;
use App\Installation\InstructionManual;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class InitialiseProject
{
    /**
     * @var \LaravelZero\Framework\Commands\Command
     */
    private $output;

    /**
     * @var \App\Installation\InstructionManual
     */
    private $instructions;

    /**
     * @var array
     */
    private $failedCommands;

    public function __construct(Command $output, InstructionManual $instructions)
    {
        $this->output = $output;
        $this->instructions = $instructions;
    }

    public static function run(string $projectName, Command $output, InstructionManual $instructions)
    {
        return (new self($output, $instructions))($projectName);
    }

    public function __invoke(string $projectName)
    {
        $commands = $this->instructions->read();

        if ($commands === null) {
            $this->output->error('Could not find any instructions to run for this project.');

            return;
        }

        $this->failedCommands = [];

        $this->output->line('');
        $this->output->comment('Detected a ' . $commands->name() . ' project');
        $this->output->setProcessTitle('Running initialisation commands');

        $this->output->getOutput()->writeln('');

        foreach ($commands as $command) {
            $command = value($command);

            if ($command === null) {
                continue;
            }

            if ($command instanceof RequiresInput) {
                $command->setOutput($this->output);
                $this->output->info('Running ' . $command->getCommand());

                if ($command->execute() === false) {
                    $this->failedCommands[] = $command;
                }

                $this->output->clear();

                continue;
            }

            $this->output->task($command->getCommand(), function () use ($command) {
                if ($command->execute() === false) {
                    $this->failedCommands[] = $command;
                    $this->output->clear();

                    return false;
                }

                $this->output->clear();

                return true;
            }, 'running');
        }

        $this->output->notify(
            'Finished setting up ' . $projectName,
            $this->getNotificationBody(),
            resource_path('icon.png')
        );

        if (blank($this->failedCommands)) {
            $this->output->getOutput()->success('Successfully set up your project!');

            return;
        }

        Storage::disk('project')->makeDirectory('drpg-installer-errors');
        foreach ($this->failedCommands as $command) {
            Storage::disk('project')->put(
                'drpg-installer-errors/' . Str::slug($command->getCommand()) . '.log',
                $command->getErrorOutput() ?: $command->getOutput()
            );
        }

        $this->output->getOutput()->block(
            [
                'Failed to run some commands. We have saved their output to',
                Storage::disk('project')->path('drpg-installer-errors/{command}.log'),
            ],
            '',
            'error',
            ' ',
            true
        );
    }

    private function getNotificationBody(): string
    {
        if (filled($this->failedCommands)) {
            return count($this->failedCommands)
                . ' '
                . Str::plural('command', count($this->failedCommands))
                . ' failed to run';
        }

        return 'There were no failed commands';
    }
}
